function [X, Y] = apply_transform(x, y, P, direction)
% CALC_TRANSFORM finds a transformation matrix that maps points [x, y] to
% points [X, Y] so that [X; Y; W] = P * [x; y; w]
%
% Usage:
%   [X, Y] = apply_transform(x, y, P)
%   [X, Y] = apply_transform(x, y, P, direction)
% Inputs:
%   x, y = vectors of x and y coordinates pre-transform 
%   P = 3x3 transformation matrix
%   direction = string, 'fwd' or 'inv'; if not provided will assume 'fwd'
% Outputs:
%   X, Y = vectors of transformed coordinates

if nargin < 4
    direction = 'fwd';
end

if strcmp(direction, 'fwd')
    % apply forward transform
    XYW = P * [x(:)'; y(:)'; ones(1, numel(x))];
    X = XYW(1, :) ./ XYW(3, :);
    Y = XYW(2, :) ./ XYW(3, :);
elseif strcmp(direction, 'inv')
    % apply inverse transform
    XYW = P \ [x(:)'; y(:)'; ones(1, numel(x))];
    X = XYW(1, :) ./ XYW(3, :);
    Y = XYW(2, :) ./ XYW(3, :);
else
    error([mfilename,':input'], 'Direction can be ''fwd'' or ''inv''.');
end
