function P = calc_transform(X, Y, x, y)
% CALC_TRANSFORM finds a transformation matrix that maps points [x, y] to
% points [X, Y] so that [X; Y; W] = P * [x; y; w]
%
% Usage:
%   P = calc_transform(X, Y, x, y)
%   P = calc_transform(X, Y)
% Inputs:
%   x, y = vectors of x and y coordinates pre-transform (optional)
%       if not provided, function will assume pre-transform points are the
%       four corners of a unit square
%   X, Y = vectors of x and y coordinates post-transform
% Outputs:
%   P = 3x3 transformation matrix

n = numel(X); % get number of points

if nargin < 4 % if pre-transform points were not provided, assume corners
    x = [0, 1, 0, 1];  %  corner format 3  4
    y = [0, 0, 1, 1];  %                1  2
end
w = ones(1, n);
xyw = [x(:)'; y(:)'; w]';
% solve for P
XY = [X(:)'; Y(:)'];
A = zeros(2*n, 8);
A(2*(1:n)-1, 1:3) = xyw;
A(2*(1:n), 4:6) = xyw;
A(:, 7) = -reshape(repmat(x(:)', 2, 1) .* XY, 2*n, 1);
A(:, 8) = -reshape(repmat(y(:)', 2, 1) .* XY, 2*n, 1);
P = A \ XY(:);
P = reshape([P; 1], 3, 3)';

