function line_ok = choose_lines(lines)

n_lines = numel(lines);
intersections = nan(n_lines-1, 2, n_lines-1);
for ind1 = 1:n_lines-1
    theta1 = lines(ind1).theta * pi/180;
    rho1 = lines(ind1).rho;
    for ind2 = ind1+1:n_lines
        theta2 = lines(ind2).theta * pi/180;
        rho2 = lines(ind2).rho;
        A = [cos(theta1), sin(theta1);
            cos(theta2), sin(theta2)];
        b = [rho1; rho2];
        x = A \ b;
        intersections(ind2-1, :, ind1) = x';
    end
end

inf_inds = isinf(intersections);
n_inf_pairs = sum(inf_inds(:))/2;
if n_inf_pairs >= n_lines/2
    all_thetas = [lines.theta];
    line_ok = all_thetas == mode(all_thetas);
    return;
end
all_pairs = reshape(intersections, n_lines-1, []);
all_pairs = reshape(all_pairs', 2, [])';
ignore_inds = isnan(all_pairs(:, 1)) | isinf(all_pairs(:, 1));
med_pt = median(all_pairs(~ignore_inds, :));

% figure('Color', 'w'); 
% gca; set(gca, 'YDir', 'reverse'); grid on; hold on;
% cmap = colormap('lines');
% h_legend = nan(n_lines, 1);
med_dist = zeros(n_lines-1, n_lines);
for iline = 1:n_lines
    these_points = nan(n_lines-1, 2);
    if iline < n_lines
        these_points(iline:end, :) = intersections(iline:end, :, iline);
    end
    if iline > 1
        these_points(1:iline-1, :) = [squeeze(intersections(iline-1, 1, 1:iline-1)), ...
            squeeze(intersections(iline-1, 2, 1:iline-1))];
    end
    med_dist(:, iline) = sqrt(sum((these_points - repmat(med_pt, n_lines-1, 1)).^2, 2));
%     h_legend(iline) = plot(these_points(:, 1), these_points(:, 2), ...
%         'Color', cmap(iline, :), 'Marker', 'o', 'MarkerSize', 2*iline+3, ...
%         'LineStyle', 'none');
end
% legend(h_legend, num2str((1:n_lines)'));

ignore_inds = isinf(med_dist) | isnan(med_dist);
dist_thresh = mean(med_dist(~ignore_inds));
n_close_ints = sum(med_dist < dist_thresh);
line_ok = n_close_ints >= min(4, n_lines/2);
