function squares = classify_squares(board_r, board_g, squares)

buffer = 10; % don't include this number of pixels from edge of square
r_g_thresh = 0.8; % percentage of foreground pixels to determine color
for irow = 1:8 
    row_inds = (irow-1)*100 + (buffer+1:100-buffer);
    for icol = 1:8
        col_inds = (icol-1)*100 + (buffer+1:100-buffer);
        % get the area within this square in the red image and green image
        area_r = board_r(row_inds, col_inds);
        area_g = board_g(row_inds, col_inds);
        % find total number of foreground pixels in red and green combined
        total = sum(area_r(:)) + sum(area_g(:));
        % if there are no foreground pixels... ?
        if total == 0
            squares(irow, icol).color = '?';
        else % otherwise, 
            % get fraction of foreground pixels that are red and green
            r_frac = sum(area_r(:))/total;
            g_frac = sum(area_g(:))/total;
            % classify square as red, green, or unknown
            if r_frac > r_g_thresh 
                squares(irow, icol).color = 'r';
            elseif g_frac > r_g_thresh
                squares(irow, icol).color = 'g';
            else
                squares(irow, icol).color = '?';
            end
        end
        fprintf(1, '%s\t', squares(irow, icol).color);
    end
    fprintf(1, '\n');
end

% fill in any unknowns
colors = reshape([squares.color], 8, 8);
is_r = colors == 'r';
is_g = colors == 'g';
have_result = is_r | is_g;
no_result_inds = find(~have_result);
if any(no_result_inds)
    fprintf(1, '%n squares were not matched with a color. Filling in ''?''s...', ...
        numel(no_result_inds));
    for isquare = no_result_inds(:)'
        [irow, icol] = ind2sub([8, 8], isquare);
        % check color of adjacent squares
        next_inds = [irow, icol-1; irow, icol+1; irow-1, icol; irow+1, icol];
        out_of_bounds = next_inds < 1 | next_inds > 8;
        next_inds(any(out_of_bounds, 2), :) = [];
        neighbor_color = colors(sub2ind([8, 8], next_inds(:, 1), next_inds(:, 2)));
        unk_inds = ismember(neighbor_color, '?');
        neighbor_color(unk_inds) = [];
        % if all adjacent squares have the same color, set this one opposite
        if numel(unique(neighbor_color)) == 1
            colors(isquare) = setdiff('rg', neighbor_color(1));
            squares(irow, icol).color = colors(isquare);
        else % otherwise... throw an error
            error([mfilename, ':incomplete'], ...
                'Contradicting square colors around (%d, %d).', irow, icol);
        end
    end
end

% do a quick check to make sure the alternating pattern is correct
% (this part assumes that all '?'s have been filled in)
odd_inds = 1:2:7;
even_inds = 2:2:8;
% start by checking first row for alternating red/green pattern
first_row_r = is_r(1, :);
first_row_g = is_g(1, :);
first_val_r = first_row_r(1);
first_row_ok = (first_row_r(odd_inds) == first_val_r) & ...
    (first_row_g(odd_inds) == ~first_val_r) & ...
    (first_row_g(even_inds) == first_val_r) & ...
    (first_row_r(even_inds) == ~first_val_r);
if any(~first_row_ok)
    error([mfilename, ':check'], 'Contradicting square colors in first row.');
end
% now make sure alternating rows folllow same pattern
all_rows_ok = is_r(odd_inds, :) == repmat(first_row_r, 4, 1)& ...
    is_g(odd_inds, :) == repmat(~first_row_r, 4, 1) & ...
    is_r(even_inds, :) == repmat(~first_row_r, 4, 1) & ...
    is_g(even_inds, :) == repmat(first_row_r, 4, 1);
if any(~all_rows_ok(:))
    error([mfilename, ':check'], 'Contradicting square colors found.');
end
% rows_ok = all(all_rows_ok, 2)

