function img = keep_area_near_line(slope, xy, img, buffer, color)
% KEEP_AREA_NEAR_LINE blacks out parts of an image outside of a buffer zone
% around a line defined by a slope and a point
% 
% Usage:
%   img = keep_area_near_line(slope, xy, img, buffer)
% Inputs:
%   slope = slope of the line
%   xy = 1x2 array with x and y coordinate of a point on the line
%   img = image to be modified
%   buffer = number of pixels above/below the line to keep; can be a scalar
%       to use the same value for above and below or a 1x2 array to use
%       different values; [b Inf] will keep all points below, and [Inf b]
%       will keep all points above

if numel(buffer) == 1
    buffer = buffer * [1, 1];
end
if nargin < 5
    color = 0;
end

img_size = size(img);
if slope == 0
    start_pt = [1, xy(1, 2)];
elseif xy(1, 1) - xy(1, 1)/slope < 0
    start_pt = [ceil(xy(1, 1) - xy(1, 2)/slope), 1];
elseif xy(1, 1) - xy(1, 1)/slope > img_size(1)
    start_pt = [ceil(xy(1, 1) + (img_size(1) - xy(1, 2))/slope), img_size(1)];
else
    start_pt = [1, ceil(xy(1, 2) - xy(1,1)*slope)];
end

y = 1:img_size(1);
for icol = 1:img_size(2)
    col_inds = false(1, img_size(2)); col_inds(icol) = true;
    if ~isinf(buffer(1))
        row_inds = slope*(icol - start_pt(1)) + start_pt(2) > y + buffer(1);
    else
        row_inds = false(1, img_size(1));
    end
    if ~isinf(buffer(2))
        row_inds = row_inds | slope*(icol - start_pt(1)) + start_pt(2) < y - buffer(2);
    end
    img(row_inds, col_inds, :) = color;
end
