function bw=silhouette(img, canny_thresh, canny_sigma,SE_size)

% This function produces the silhouette shape of the chess piece
% it uses Canny edge detector to get the shape of the piece instead of
% Otsu algorithm to avoid non uniform lighting problems
% To use default values of the canny parameters of the SE size enter them
% as []
%%%%%%Inputs%%%%%%%%%%%
% img: color or gray image of a chess piece
% canny_thresh: the lower and upper thresholds for canny edge detector...
%               in any order (between 0 and 1) default=[0.1,0.2]
% canny_sigma: sigma of canny edge detector (default= 1)
% SE_size: size of square SE for dilation (default=5)

if isempty(canny_thresh)
    canny_thresh=[0.1,0.2];
end
if isempty(canny_sigma)
    canny_sigma=1;
end
if isempty(SE_size)
    SE_size=5;
end
    
if (size(img,3)>1)
gray_img=rgb2gray(img);
else
    gray_img=img;
end
[height,width]=size(gray_img);
figure;imshow(gray_img)
edge_img=edge(gray_img.^0.5,'canny',[min(canny_thresh), max(canny_thresh)],canny_sigma);
figure;imshow(edge_img);
w=strel('square',SE_size);
edge_img=imdilate(edge_img,w);
figure;imshow(edge_img);
[y,x]=find(edge_img==1);
bw=imfill(edge_img,'holes');
figure;imshow(bw);
[L,n]=bwlabel(bw);
figure;imshow(label2rgb(L));
for i=1:n
    [r,c]=find(L==i);
    if(length(r)<10000)
        bw(L==i)=0;
    end
end
end