function [squares,connected_regions] = straighten_board(img_data, squares, connected_regions, label)

% put all of the corner values into a single array
xy = nan(81, 2);
ind = 1;
for isquare = 1:numel(squares)
    if mod(isquare, 8) == 1
        xy(ind, :) = squares(isquare).corners(1, :); 
        ind = ind+1;
    end
    xy(ind, :) = squares(isquare).corners(3, :);
    ind = ind+1;
end
xy(ind, :) = squares(57).corners(2, :);
ind = ind+1;
for isquare = 57:64
    xy(ind, :) = squares(isquare).corners(4, :);
    ind = ind+1;
end

% find homography transformation matrix
[a, b] = meshgrid(0:100:800);
P = calc_transform(xy(:, 1), xy(:, 2), a(:), b(:));

% apply transformation to red and green binary images separately
board_r = process_binary_img(img_data.img_r, xy, P);
board_g = process_binary_img(img_data.img_g, xy, P);
% decide if each square is red or green
squares = classify_squares(board_r, board_g, squares);

% if connected_regions was provided, determine occupancy of squares
if nargin > 2
    n_pieces = numel(connected_regions);
    for ipiece = 1:n_pieces
        this_piece = connected_regions(ipiece);
        x = this_piece.base_xposition;
        y = this_piece.base_yposition;
        [straight_x, straight_y] = apply_transform(x, y, P, 'inv');
        if straight_x < 0 || straight_y < 0 || straight_x > 800 || straight_y > 800
            connected_regions(ipiece).color='n';
            label(label==ipiece)=0;
            continue; 
        end
    end 
    all_gray_vals = img_data.img_gray(label > 0);
    gray_thresh = graythresh(all_gray_vals);
    buffer = 50;
    for ipiece = 1:n_pieces
        this_piece = connected_regions(ipiece);
        x = this_piece.base_xposition;
        y = this_piece.base_yposition;
        [straight_x, straight_y] = apply_transform(x, y, P, 'inv');
        if straight_x < 0 || straight_y < 0 || straight_x > 800 || straight_y > 800
            connected_regions(ipiece).color='n';
            label(label==ipiece)=0;
            continue; 
        end

        upper_x = 100*ceil(straight_x/100);
        upper_y = 100*ceil(straight_y/100);
        center = [upper_x - 50, upper_y - 50];
        deltas = abs([straight_x, straight_y] - center);
        if all(deltas < buffer)
            irow = 9-upper_y/100;
            icol = upper_x/100;
            squares(irow, icol).class = this_piece.class;
            
            inds = label == ipiece;
            gray_vals = img_data.img_gray(inds);
            if median(gray_vals) < gray_thresh*255
                squares(irow, icol).occupancy = 'black';
                connected_regions(ipiece).color='b';
            else
                squares(irow, icol).occupancy = 'white';
                connected_regions(ipiece).color='w';
            end
        end
    end
end

% ---
function img_straight = process_binary_img(img_gr, xy, P)

% crop image to have only area within bounding box of corners
min_x = floor(min(xy(:, 1)));
min_y = floor(min(xy(:, 2)));
max_x = min(ceil(max(xy(:, 1))), size(img_gr, 2));
max_y = min(ceil(max(xy(:, 2))), size(img_gr, 1));
bw_crop = img_gr(min_y:max_y, min_x:max_x);

% img_straight = nan(800, 800);
if islogical(bw_crop)
    val_array = true;
    img_straight = false(800, 800);
else
    val_array = uint8(0:255);
    img_straight = zeros(800, 800, 'uint8');
end
for ival = 1:numel(val_array)
    val = val_array(ival);
    inds = find(bw_crop(:) == val);
    % convert image into x and y coordinates of true/white pixels
    [y2, x2] = ind2sub(size(bw_crop), inds);
    x2 = x2' + min_x; % add offset back in so transform will work
    y2 = y2' + min_y;

    [X2, Y2] = apply_transform(x2, y2, P, 'inv');
    bad_inds = X2 <= 0 | Y2 <= 0 | X2 > 800 | Y2 > 800;
    X2(bad_inds) = []; 
    Y2(bad_inds) = [];

    ind = sub2ind(size(img_straight), ceil(Y2), ceil(X2));
    img_straight(ind) = val;
end

%figure('Color', 'w');
% if islogical(bw_crop)
img_straight = imclose(img_straight, ones(3));
img_straight = flipud(img_straight);
% imshow(img_straight, 'InitialMagnification', 'fit');
% else
%     img_to_show = img_straight;
%     img_to_show(isnan(img_to_show)) = 0;
%     img_to_show = uint8(img_to_show);
%     imshow(img_to_show, 'InitialMagnification', 'fit');
% end

