function [connected_regions_vote, connected_regions_sideinfo]= vote_sideinfo2(CR,Label,numregions,option,num_pcs)

connected_regions_vote=CR{1};
connected_regions_sideinfo=[];
match=match_regions(CR);
[classify,error]=vote(match,option);
figure; imshow(label2rgb(Label{1}));
for i=1:numregions{1}
    [r,c]=find(Label{1}==i);
    text(round(c(round(length(r)/2))),round(r(round(length(r)/2))),[num2str(i),classify(i)],'color','k');
    hold on
    connected_regions_vote(i).class=classify(i);
    connected_regions_vote(i).distance=error(i);
end
if (~isempty(num_pcs))
    connected_regions_sideinfo=connected_regions_vote;
    for i=1:numel(connected_regions_vote)
        color(i)=connected_regions_vote(i).color;
    end
reclassify=side_info(num_pcs,match,classify,error,color);
figure; imshow(label2rgb(Label{1})); hold on
for i=1:numregions{1}
    [r,c]=find(Label{1}==i);
    text(round(c(round(length(r)/2))),round(r(round(length(r)/2))),[num2str(i),reclassify(i)],'color','k');
    hold on
    connected_regions_sideinfo(i).class=reclassify(i);
end
end
end
function match=match_regions(connected_regions)

%%% Usage: This function matches connected regions of different binary images of
%%%        same color or gray image
%%% Input: connected_regions cell array. Each cell is the output of the
%%%        "Recognize_pcs" function
%%%         The function finds a match for each connected region in the
%%%         first cell from the other cells.
%%% Output: match is a structure array (with length equals the number of
%%% connected regions in image 1. 
%%%         match.img_idx array of matched images
%%%         match.region_idx matches regions 
%%%         matc.class shows the classification of the region in different
%%%         matched images

num_voters=numel(connected_regions);
for voter=1:num_voters
    num_regions(voter)=numel(connected_regions{voter});
    for region=1:num_regions(voter)
         centroid{voter}(region,:)=connected_regions{voter}(region).centroid;
    end
end
voter=1;
    for region=1:num_regions(voter)
        match(region).img_idx=[];
        match(region).region_idx=[];
        match(region).class=[];
        match(region).distance=[];
        %%%match each region to other regions in other images
        for other_voters=2:num_voters
            centdist_error=centroid{other_voters}-repmat(centroid{voter}(region,:),num_regions(other_voters),1);
            centdist_error=norms(centdist_error,2,2);
[centdist,centdist_ind]=min(centdist_error);
            if(centdist<20)
                match(region).img_idx=[match(region).img_idx,other_voters];
                match(region).region_idx=[match(region).region_idx,centdist_ind];
                match(region).class=[match(region).class,connected_regions{other_voters}(centdist_ind).class];
                match(region).distance=[match(region).distance,connected_regions{other_voters}(centdist_ind).distance];
            end
%            Recognition(region,other_voters)=connected_regions{other_voters}(baseposdist_ind(voter,other_voters)).class;
        end
        match(region).img_idx=[1,match(region).img_idx];
        match(region).region_idx=[region,match(region).region_idx];
        match(region).class=[connected_regions{voter}(region).class,match(region).class];
        match(region).distance=[connected_regions{voter}(region).distance,match(region).distance];
end
    
end
    function [classify,error]=vote(match,option)

%%% Usage: This function suggests a classification for each connected region 
%%% Input: match (the output of "match_regions" function
%%% Input: option determines type of decision if option='majority' each
%%% region is classified based on the majority of votes. If
%%% option='mindist', each region is classified according to the minimum
%%% distance of descriptors.
%%% Output: array of characters of length equals the number of connected
%%% regions and character indicating the classification of each region
num_regions=numel(match);

if strcmp(option,'majority')
    names=['G','Q','B','P','R','K'];
for region=1:num_regions
        class_counter=zeros(1,6);
    candidate_classes=match(region).class;
    for i=1:6
        indices=find(candidate_classes==names(i));
        class_counter(i)=class_counter(i)+numel(indices);
    end
    [maxv,maxind]=max(class_counter);
    classify(region)=names(maxind);
    error(region)=match(region).distance(find(match(region).class==classify(region),1));
end
elseif strcmp(option,'mindist')
    for region=1:num_regions
        distances=match(region).distance;
        index=find(distances==min(distances),1);
        classify(region)=match(region).class(index);
        error(region)=min(match(region).distance);
    end   
end
    end
function classify=side_info(num_pcs,match,classify,error,color)

%%% Use: use knowledge of the number of pieces on the board to refine
%%%      classification
%%% Input: "num_pcs" : is a vector of number of pcs of each type in the
%%%        following order [king,queen,bishop,pawn,rook,knight]
%%%        "match" : is the output of 'match_regions' function
%%%         "classify" and "error" are the output of 'vote' function


% pcs_counter=zeros(1,6);
names=['G','Q','B','P','R','K'];
num_regions=numel(match);
for i=1:6
indices=find(classify==names(i)& color~='n');
pcs_idx{i}=indices;
pcs_counter(i)=numel(indices);
end
excess_pcs=pcs_counter-num_pcs;
plus=find(excess_pcs>0); minus=find(excess_pcs<0);
extra=excess_pcs(plus);
if length(plus)==1 && length(minus)==1
    for i=1:extra
    [extra_error, extra_ind]=max(error(pcs_idx{plus}));
    classify(pcs_idx{plus}(extra_ind))=names(minus);
    pcs_idx{minus}=[pcs_idx{minus},pcs_idx{plus}(extra_ind)];
    pcs_idx{plus}(extra_ind)=[];
    end
else
empty=ones(1,6);
history=cell(num_regions,1);
for i=1:num_regions
    history{i}=classify(i);
end
while (1)
for i=1:6
    if excess_pcs(i)>0
        error_vec=error(pcs_idx{i});
        maxind=find(error_vec==max(error_vec));
        corrected_region=pcs_idx{i}(maxind);
        a=ones(1,numel(match(corrected_region).class));
        for j=1:numel(history{corrected_region})
            a= a & (match(corrected_region).class~=history{corrected_region}(j));
        end
        %newind=find(match(corrected_region).class~=names(i),1);
        newind=find(a,1);
        if (~isempty(newind))
        classify(corrected_region)=match(corrected_region).class(newind);
        history{corrected_region}=[history{corrected_region},classify(corrected_region)];
        indexx=find(names==classify(corrected_region));
        excess_pcs(indexx)=excess_pcs(indexx)+1;
        excess_pcs(i)=excess_pcs(i)-1;
        error(corrected_region)=match(corrected_region).distance(newind);
        pcs_idx{indexx}=[pcs_idx{indexx},corrected_region];
        pcs_idx{i}(maxind)=[];
        else
            empty(i)=0;
        end
    end
end
if (sum(abs(excess_pcs))==0 || sum(empty(excess_pcs>0))==0)
    break
end
end
end
end



